from tkinter import *
from tkinter.tix import *
from datetime import *
from read_csv import read_csv
import numpy as np


def show_timetable(current_schedule):

    root = Tk()

    # set title and size of the GUI window
    root.title('Timetable')
    root.geometry("1200x700+350+150")

    # set secondary frame position and size (center right)
    frame = Frame(root)
    frame.place(relx=0.05, rely=0.05, relwidth=0.9, relheight=0.9)

    # get today's date and set as heading
    today = date.today()
    if today.weekday() == 5:
        today = date.today() - timedelta(days=1)
    if today.weekday() == 6:
        today = date.today() - timedelta(days=2)
    week_day = today.strftime("%A")

    # place main date label
    heading = Label(frame, text= week_day + " , " + str(today), bg='#E5E7E9', relief=RIDGE)
    heading.place(relx=0.38, relwidth=0.25, y=0)

    # set secondary frame position and size
    canvas = Canvas(root)
    canvas.place(relx=0.1, rely=0.1, relwidth=0.8, relheight=0.8)

    # create a frame inside the canvas
    canvas_frame = Frame(canvas, width=955, height=1500)
    canvas_frame.pack(expand=True, fill=BOTH)

    canvas_window = canvas.create_window(0, 0, window=canvas_frame, anchor="nw")

    # create and set vertical scrollbar
    vbar = Scrollbar(frame, orient=VERTICAL, command=canvas.yview)
    vbar.pack(side=RIGHT, fill=Y)

    canvas.config(yscrollcommand=vbar.set)

    # time-table headings
    label1 = Label(canvas_frame, text='Time:', bg='#D0ECE7', relief=RIDGE)
    label2 = Label(canvas_frame, text='L1', bg='#D0ECE7', relief=RIDGE)
    label3 = Label(canvas_frame, text='L2', bg='#D0ECE7', relief=RIDGE)
    label4 = Label(canvas_frame, text='L3', bg='#D0ECE7', relief=RIDGE)
    label5 = Label(canvas_frame, text='L4', bg='#D0ECE7', relief=RIDGE)
    label6 = Label(canvas_frame, text='L5', bg='#D0ECE7', relief=RIDGE)
    label7 = Label(canvas_frame, text='L6', bg='#D0ECE7', relief=RIDGE)
    label8 = Label(canvas_frame, text='L7', bg='#D0ECE7', relief=RIDGE)
    label9 = Label(canvas_frame, text='L8', bg='#D0ECE7', relief=RIDGE)
    label10 = Label(canvas_frame, text='M1', bg='#D0ECE7', relief=RIDGE)
    label11 = Label(canvas_frame, text='M2', bg='#D0ECE7', relief=RIDGE)
    label12 = Label(canvas_frame, text='M3', bg='#D0ECE7', relief=RIDGE)
    label13 = Label(canvas_frame, text='M4', bg='#D0ECE7', relief=RIDGE)
    label14 = Label(canvas_frame, text='M5', bg='#D0ECE7', relief=RIDGE)
    label15 = Label(canvas_frame, text='OT 24', bg='#D0ECE7', relief=RIDGE)
    label16 = Label(canvas_frame, text='OT 25', bg='#D0ECE7', relief=RIDGE)
    label17 = Label(canvas_frame, text='OT 22', bg='#D0ECE7', relief=RIDGE)
    label18 = Label(canvas_frame, text='R1', bg='#D0ECE7', relief=RIDGE)
    label19 = Label(canvas_frame, text='R4', bg='#D0ECE7', relief=RIDGE)
    label20 = Label(canvas_frame, text='R5', bg='#D0ECE7', relief=RIDGE)
    label21 = Label(canvas_frame, text='R6', bg='#D0ECE7', relief=RIDGE)
    label22 = Label(canvas_frame, text='R7', bg='#D0ECE7', relief=RIDGE)
    label23 = Label(canvas_frame, text='R8', bg='#D0ECE7', relief=RIDGE)
    label24 = Label(canvas_frame, text='MRI', bg='#D0ECE7', relief=RIDGE)
    label1.place(relwidth=0.08)
    label2.place(relwidth=0.04, relx=0.08)
    label3.place(relwidth=0.04, relx=0.12)
    label4.place(relwidth=0.04, relx=0.16)
    label5.place(relwidth=0.04, relx=0.20)
    label6.place(relwidth=0.04, relx=0.24)
    label7.place(relwidth=0.04, relx=0.28)
    label8.place(relwidth=0.04, relx=0.32)
    label9.place(relwidth=0.04, relx=0.36)
    label10.place(relwidth=0.04, relx=0.40)
    label11.place(relwidth=0.04, relx=0.44)
    label12.place(relwidth=0.04, relx=0.48)
    label13.place(relwidth=0.04, relx=0.52)
    label14.place(relwidth=0.04, relx=0.56)
    label15.place(relwidth=0.04, relx=0.60)
    label16.place(relwidth=0.04, relx=0.64)
    label17.place(relwidth=0.04, relx=0.68)
    label18.place(relwidth=0.04, relx=0.72)
    label19.place(relwidth=0.04, relx=0.76)
    label20.place(relwidth=0.04, relx=0.80)
    label21.place(relwidth=0.04, relx=0.84)
    label22.place(relwidth=0.04, relx=0.88)
    label23.place(relwidth=0.04, relx=0.92)
    label24.place(relwidth=0.04, relx=0.96)

    # time column labels
    time1 = Label(canvas_frame, text='0800:0815', bg='#FDFEFE', relief=RIDGE)
    time2 = Label(canvas_frame, text='0815:0830', bg='#FDFEFE', relief=RIDGE)
    time3 = Label(canvas_frame, text='0830:0845', bg='#FDFEFE', relief=RIDGE)
    time4 = Label(canvas_frame, text='0845:0900', bg='#FDFEFE', relief=RIDGE)
    time5 = Label(canvas_frame, text='0900:0915', bg='#FDFEFE', relief=RIDGE)
    time6 = Label(canvas_frame, text='0915:0930', bg='#FDFEFE', relief=RIDGE)
    time7 = Label(canvas_frame, text='0930:0945', bg='#FDFEFE', relief=RIDGE)
    time8 = Label(canvas_frame, text='0945:1000', bg='#FDFEFE', relief=RIDGE)
    time9 = Label(canvas_frame, text='1000:1015', bg='#FDFEFE', relief=RIDGE)
    time10 = Label(canvas_frame, text='1015:1030', bg='#FDFEFE', relief=RIDGE)
    time11 = Label(canvas_frame, text='1030:1045', bg='#FDFEFE', relief=RIDGE)
    time12 = Label(canvas_frame, text='1045:1100', bg='#FDFEFE', relief=RIDGE)
    time13 = Label(canvas_frame, text='1100:1115', bg='#FDFEFE', relief=RIDGE)
    time14 = Label(canvas_frame, text='1115:1130', bg='#FDFEFE', relief=RIDGE)
    time15 = Label(canvas_frame, text='1130:1145', bg='#FDFEFE', relief=RIDGE)
    time16 = Label(canvas_frame, text='1145:1200', bg='#FDFEFE', relief=RIDGE)
    time17 = Label(canvas_frame, text='1200:1215', bg='#FDFEFE', relief=RIDGE)
    time18 = Label(canvas_frame, text='1215:1230', bg='#FDFEFE', relief=RIDGE)
    time19 = Label(canvas_frame, text='1230:1245', bg='#FDFEFE', relief=RIDGE)
    time20 = Label(canvas_frame, text='1245:1300', bg='#FDFEFE', relief=RIDGE)
    time21 = Label(canvas_frame, text='1300:1315', bg='#FDFEFE', relief=RIDGE)
    time22 = Label(canvas_frame, text='1315:1330', bg='#FDFEFE', relief=RIDGE)
    time23 = Label(canvas_frame, text='1330:1345', bg='#FDFEFE', relief=RIDGE)
    time24 = Label(canvas_frame, text='1345:1400', bg='#FDFEFE', relief=RIDGE)
    time25 = Label(canvas_frame, text='1400:1415', bg='#FDFEFE', relief=RIDGE)
    time26 = Label(canvas_frame, text='1415:1430', bg='#FDFEFE', relief=RIDGE)
    time27 = Label(canvas_frame, text='1430:1445', bg='#FDFEFE', relief=RIDGE)
    time28 = Label(canvas_frame, text='1445:1500', bg='#FDFEFE', relief=RIDGE)
    time29 = Label(canvas_frame, text='1500:1515', bg='#FDFEFE', relief=RIDGE)
    time30 = Label(canvas_frame, text='1515:1530', bg='#FDFEFE', relief=RIDGE)
    time31 = Label(canvas_frame, text='1530:1545', bg='#FDFEFE', relief=RIDGE)
    time32 = Label(canvas_frame, text='1545:1600', bg='#FDFEFE', relief=RIDGE)
    time33 = Label(canvas_frame, text='1600:1615', bg='#FDFEFE', relief=RIDGE)
    time34 = Label(canvas_frame, text='1615:1630', bg='#FDFEFE', relief=RIDGE)
    time35 = Label(canvas_frame, text='1630:1645', bg='#FDFEFE', relief=RIDGE)
    time36 = Label(canvas_frame, text='1645:1700', bg='#FDFEFE', relief=RIDGE)
    time37 = Label(canvas_frame, text='1700:1715', bg='#FDFEFE', relief=RIDGE)
    time38 = Label(canvas_frame, text='1715:1730', bg='#FDFEFE', relief=RIDGE)
    time39 = Label(canvas_frame, text='1730:1745', bg='#FDFEFE', relief=RIDGE)
    time40 = Label(canvas_frame, text='1745:1800', bg='#FDFEFE', relief=RIDGE)
    time41 = Label(canvas_frame, text='1800:1815', bg='#FDFEFE', relief=RIDGE)
    time42 = Label(canvas_frame, text='1815:1830', bg='#FDFEFE', relief=RIDGE)
    time43 = Label(canvas_frame, text='1830:1845', bg='#FDFEFE', relief=RIDGE)
    time44 = Label(canvas_frame, text='1845:1900', bg='#FDFEFE', relief=RIDGE)

    # placing all title for the time column
    for i in range(1, 45):
        vars()['time' + str(i)].place(relwidth=0.08, y=20 + (i - 1) * 20, )
        i = i + 1

    # create time-table using labels (44rows*23columns)
    rows = 44
    cols = 24
    blk = [[0 for i in range(cols)] for j in range(rows)]
    for i in range(rows):  # Rows
        for j in range(cols):  # Columns
            blk[i][j] = Label(canvas_frame, text="", bg='#FDFEFE', relief=RIDGE)
            blk[i][j].place(relwidth=0.04, relx=0.08 + j * 0.04, y=20 + i * 20)

    color_code = {'OTO':'#D35400', 'H&N':'#D7BDE2', 'CLR':'#27AE60', 'HPB':'#F4ECF7', 'SUR-ONCO':'#E91E63',
                  'BREAST':'#00FF33', 'ENT':'#FFF59D', 'NES':'#FFF59D', 'VAS':'#E0F7FA', 'O&G':'#757575',
                  'HND':'#E0E0E0', 'UGI':'#006600', 'PLS':'#F5F5F5', 'CTS':'#A569BD', 'URO':'#5DADE2',
                  'OMS':'#A9CCE3', 'Actual':'#3F51B5', 'Predicted':'#34495E'}

    ot_dict = {'L1': 0, 'L2': 1, 'L3': 2, 'L4': 3, 'L5': 4, 'L6': 5, 'L7': 6, 'L8': 7, 'M1': 8, 'M2':9, 'M3': 10, 'M4': 11, 'M5': 12, 'OT 24': 13,
               'OT 25': 14, 'OT 22': 15, 'R1': 16, 'R4': 17, 'R5': 18, 'R6': 19, 'R7': 20, 'R8': 21, 'MRI': 22}

    discipline_codes = {10: 'BREAST',
                        11: 'OTO',
                        12: 'NES',
                        13: 'CLR',
                        14: 'HPB',
                        15: 'PLS',
                        16: 'ENT',
                        17: 'SUR-ONCO',
                        18: 'UGI',
                        19: 'H&N',
                        20: 'O&G',
                        21: 'HND',
                        22: 'OMS',
                        23: 'VAS',
                        24: 'CTS',
                        25: 'URO'
                        }

    proc_codes_oto = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    # find first date found in current schedule and set it as date appearing in timetable
    first_day_from_now = current_schedule[1][0]
    today = date.today()
    first_day = (today + timedelta(days=first_day_from_now)).strftime("%A")
    first_date = (today + timedelta(days=first_day_from_now))
    heading.config(text=first_day + " , " + str(first_date))

    # show timetable for first date in schedule
    col_index = 0
    for slots in current_schedule[0][0]:  # this each OT array for the first date
        nonzero_item_index = np.where(slots != 0)  # find indexes in each OT array where element is non-zero
        for j in range(len(nonzero_item_index[0])):  # access each individual index that is non-zero
            discipline_digit = int(str(slots[nonzero_item_index[0][j]])[2:4])
            procedure_digit = int(str(slots[nonzero_item_index[0][j]])[4:6])
            last_digit = int(str(slots[nonzero_item_index[0][j]])[-1])
            if str(slots[nonzero_item_index[0][j]])[4:6] == '00':
                blk[nonzero_item_index[0][j]][col_index].config(bg=color_code[discipline_codes[discipline_digit]], text=discipline_codes[discipline_digit])  # 1st index=row, 2nd index=col
            if str(slots[nonzero_item_index[0][j]])[4:6] != '00' and str(slots[nonzero_item_index[0][j]])[1] == '1':
                blk[nonzero_item_index[0][j]][col_index].config(bg=color_code['Actual'], text=proc_codes_oto[procedure_digit]+str(last_digit))
            if str(slots[nonzero_item_index[0][j]])[4:6] != '00' and str(slots[nonzero_item_index[0][j]])[1] == '2':
                blk[nonzero_item_index[0][j]][col_index].config(bg=color_code['Predicted'], text=proc_codes_oto[procedure_digit]+str(last_digit))
        col_index = col_index+1

    # defining the prev and next function
    def prev_date():
        for i in range(rows):  # Rows
            for j in range(cols):  # Columns
                blk[i][j].config(bg='white', text='')
        str_date = heading.cget('text')
        parts = str_date.split(' , ')
        date_now = datetime.strptime(parts[1], '%Y-%m-%d').date()
        date_prev = date_now - timedelta(days=1)
        if date_prev.weekday() == 6:
            date_prev = date_prev - timedelta(days=2)
        else:
            pass
        day_prev = date_prev.strftime("%A")
        heading.config(text=str(day_prev) + " , " + str(date_prev))

        days_xx = (date_prev - date.today()).days
        # grey out current scheduled appointments
        if days_xx in current_schedule[1]:
            day_index = current_schedule[1].index(days_xx)
            col_index = 0
            for slots in current_schedule[0][day_index]:  # this each OT array for the first date
                nonzero_item_index = np.where(slots != 0)  # find indexes in each OT array where element is non-zero
                for j in range(len(nonzero_item_index[0])):  # access each individual index that is non-zero
                    discipline_digit = int(str(slots[nonzero_item_index[0][j]])[2:4])
                    procedure_digit = int(str(slots[nonzero_item_index[0][j]])[4:6])
                    last_digit = int(str(slots[nonzero_item_index[0][j]])[-1])
                    if str(slots[nonzero_item_index[0][j]])[4:6] == '00':
                        blk[nonzero_item_index[0][j]][col_index].config(bg=color_code[discipline_codes[discipline_digit]], text=discipline_codes[discipline_digit])  # 1st index=row, 2nd index=col
                    if str(slots[nonzero_item_index[0][j]])[4:6] != '00' and str(slots[nonzero_item_index[0][j]])[1] == '1':
                        blk[nonzero_item_index[0][j]][col_index].config(bg=color_code['Actual'], text=proc_codes_oto[procedure_digit] + str(last_digit))
                    if str(slots[nonzero_item_index[0][j]])[4:6] != '00' and str(slots[nonzero_item_index[0][j]])[1] == '2':
                        blk[nonzero_item_index[0][j]][col_index].config(bg=color_code['Predicted'], text=proc_codes_oto[procedure_digit] + str(last_digit))
                col_index = col_index + 1
        else:
            for i in range(rows):  # Rows
                for j in range(cols):  # Columns
                    blk[i][j].config(bg='white', text='')

    def next_date():
        for i in range(rows):  # Rows
            for j in range(cols):  # Columns
                blk[i][j].config(bg='white', text='')
        str_date = heading.cget('text')
        parts = str_date.split(' , ')
        date_now = datetime.strptime(parts[1], '%Y-%m-%d').date()
        date_next = date_now + timedelta(days=1)
        if date_next.weekday() == 5:
            date_next = date_next + timedelta(days=2)
        else:
            pass
        day_next = date_next.strftime("%A")
        heading.config(text=str(day_next) + " , " + str(date_next))
        # x = label indicated at top of GUI
        days_xx = (date_next - date.today()).days
        # grey out current scheduled appointments
        if days_xx in current_schedule[1]:
            day_index = current_schedule[1].index(days_xx)
            col_index = 0
            for slots in current_schedule[0][day_index]:  # this each OT array for the first date
                nonzero_item_index = np.where(slots != 0)  # find indexes in each OT array where element is non-zero
                for j in range(len(nonzero_item_index[0])):  # access each individual index that is non-zero
                    discipline_digit = int(str(slots[nonzero_item_index[0][j]])[2:4])
                    procedure_digit = int(str(slots[nonzero_item_index[0][j]])[4:6])
                    last_digit = int(str(slots[nonzero_item_index[0][j]])[-1])
                    if str(slots[nonzero_item_index[0][j]])[4:6] == '00':
                        blk[nonzero_item_index[0][j]][col_index].config(bg=color_code[discipline_codes[discipline_digit]], text=discipline_codes[discipline_digit])  # 1st index=row, 2nd index=col
                    if str(slots[nonzero_item_index[0][j]])[4:6] != '00' and str(slots[nonzero_item_index[0][j]])[1] == '1':
                        blk[nonzero_item_index[0][j]][col_index].config(bg=color_code['Actual'], text=proc_codes_oto[procedure_digit] + str(last_digit))
                    if str(slots[nonzero_item_index[0][j]])[4:6] != '00' and str(slots[nonzero_item_index[0][j]])[1] == '2':
                        blk[nonzero_item_index[0][j]][col_index].config(bg=color_code['Predicted'], text=proc_codes_oto[procedure_digit] + str(last_digit))
                col_index = col_index + 1
        else:
            for i in range(rows):  # Rows
                for j in range(cols):  # Columns
                    blk[i][j].config(bg='white', text='')

    prev_button = Button(frame, text='prev', width=10, bg='#E5E7E9',command=prev_date)
    next_button = Button(frame, text='next', width=10, bg='#E5E7E9',command=next_date)
    prev_button.place(relx=0.40, rely=0.95)
    next_button.place(relx=0.58, rely=0.95)

    # this section is for the creation of the scheduled surgery table
    ot_list = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'M1', 'M2', 'M3', 'M4', 'M5', 'OT 24',
               'OT 25', 'OT 22', 'R1', 'R4', 'R5', 'R6', 'R7', 'R8', 'MRI']

    discipline_codes = {10: 'BREAST',
                        11: 'OTO',
                        12: 'NES',
                        13: 'CLR',
                        14: 'HPB',
                        15: 'PLS',
                        16: 'ENT',
                        17: 'SUR-ONCO',
                        18: 'UGI',
                        19: 'H&N',
                        20: 'O&G',
                        21: 'HND',
                        22: 'OMS',
                        23: 'VAS',
                        24: 'CTS',
                        25: 'URO'
                        }

    proc_codes_breast = {10: 'mastectomy',
                         11: 'lumpectomy',
                         12: 'mastopexy',
                         13: 'biopsy',
                         14: 'mammoplasty',
                         15: 'abscess'
                         }

    proc_codes_nes = {10: 'Circumcision',
                      11: 'Pyeloplasty',
                      12: 'Nephrectomy',
                      13: 'Cystoscopy',
                      14: 'Meatoplasty',
                      15: 'Scrotoplasty'
                      }

    proc_codes_oto = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_clr = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_hpb = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_pls = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_ent = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_suronco = {10: 'hip',
                          11: 'tail',
                          12: 'ankle',
                          13: 'knee',
                          14: 'leg',
                          15: 'elbow'
                          }

    proc_codes_ugi = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_hnn = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_og = {10: 'hip',
                     11: 'tail',
                     12: 'ankle',
                     13: 'knee',
                     14: 'leg',
                     15: 'elbow'
                     }

    proc_codes_hnd = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_oms = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_vas = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_cts = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_uro = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proced_name_dict = {10: proc_codes_breast,
                        11: proc_codes_oto,
                        12: proc_codes_nes,
                        13: proc_codes_clr,
                        14: proc_codes_hpb,
                        15: proc_codes_pls,
                        16: proc_codes_ent,
                        17: proc_codes_suronco,
                        18: proc_codes_ugi,
                        19: proc_codes_hnn,
                        20: proc_codes_og,
                        21: proc_codes_hnd,
                        22: proc_codes_oms,
                        23: proc_codes_vas,
                        24: proc_codes_cts,
                        25: proc_codes_uro}

    # list to collect data of actual surgeries in the 'soln' array
    # data to retrieve from array : discipline(3-4 digits) (done), procedure(5-6 digits) (done),
    # data to retrieve from array: duplicate no.(7th or last dig) (done),
    # data to retrieve from array : dates, time start and end (done), duration (done)
    actual_surgery_proced = []
    actual_surgery_durations = []
    actual_surgery_time = []
    actual_surgery_dates = []
    actual_surgery_ot = []
    final_list = []

    soln_list = current_schedule[0]
    sched_dates = current_schedule[2]
    # sieve through each time slot in the solution list and find all actual surgeries and their allocated timeslots
    for day in range(len(soln_list)):
        for ot in range(len(soln_list[day])):
            dur_count = 0
            for time_slot in range(len(soln_list[day][ot])):
                if soln_list[day][ot][time_slot] != 0 and str(soln_list[day][ot][time_slot])[4:6] != '00' and \
                        str(soln_list[day][ot][time_slot])[1] == '1':
                    dur_count = dur_count + 1
                    # this if statement decides when to end time_slot counts (2 different criterias)
                    if (time_slot == 43 and dur_count != 0) or (str(soln_list[day][ot][time_slot]) != str(
                            soln_list[day][ot][time_slot + 1]) and dur_count != 0):
                        disc_num = int(str(soln_list[day][ot][time_slot])[2:4])
                        proced_num = int(str(soln_list[day][ot][time_slot])[4:6])
                        duplicate_num = int(str(soln_list[day][ot][time_slot])[-1])
                        duration_num = dur_count / 4.0
                        time_end_deci = 8 + (time_slot + 1) / 4
                        time_start_deci = time_end_deci - duration_num
                        time_end = str(timedelta(hours=time_end_deci)).rsplit(':', 1)[0]
                        time_start = str(timedelta(hours=time_start_deci)).rsplit(':', 1)[0]
                        actual_surgery_ot.append(ot_list[ot])
                        actual_surgery_dates.append(sched_dates[day])
                        actual_surgery_time.append(str(time_start) + ' - ' + str(time_end))
                        actual_surgery_durations.append(str(duration_num) + ' hrs')
                        actual_surgery_proced.append(proced_name_dict[disc_num][proced_num] + '_' + str(duplicate_num))
                        dur_count = 0
                else:
                    # if next timeslot is = 0, signify end of actual surgery at previous slot
                    if dur_count != 0:
                        disc_num = int(str(soln_list[day][ot][time_slot - 1])[2:4])
                        proced_num = int(str(soln_list[day][ot][time_slot - 1])[4:6])
                        duplicate_num = int(str(soln_list[day][ot][time_slot - 1])[-1])
                        duration_num = dur_count / 4.0
                        time_end_deci = 8 + (time_slot) / 4
                        time_start_deci = time_end_deci - duration_num
                        time_end = str(timedelta(hours=time_end_deci)).rsplit(':', 1)[0]
                        time_start = str(timedelta(hours=time_start_deci)).rsplit(':', 1)[0]
                        actual_surgery_ot.append(ot_list[ot])
                        actual_surgery_dates.append(sched_dates[day])
                        actual_surgery_time.append(str(time_start) + ' - ' + str(time_end))
                        actual_surgery_durations.append(str(duration_num) + ' hrs')
                        actual_surgery_proced.append(proced_name_dict[disc_num][proced_num] + '_' + str(duplicate_num))
                        dur_count = 0

    actual_surgery_proced = ['Procedure :'] + actual_surgery_proced
    actual_surgery_dates = ['Date :'] + actual_surgery_dates
    actual_surgery_ot = ['OT :'] + actual_surgery_ot
    actual_surgery_time = ['Time :'] + actual_surgery_time
    actual_surgery_durations = ['Duration :'] + actual_surgery_durations

    final_list.append(actual_surgery_proced)
    final_list.append(actual_surgery_dates)
    final_list.append(actual_surgery_ot)
    final_list.append(actual_surgery_time)
    final_list.append(actual_surgery_durations)

    rows_list = []

    # row position starts at 900
    for i in range(len(actual_surgery_time)):
        cols_list = []
        for j in range(5):
            # e = Label(canvas_frame, relief=GROOVE, width=20)
            # e.grid(row=i, column=j, sticky=NSEW)
            e = Label(canvas_frame, relief=GROOVE)
            e.place(relwidth=0.2, relx=j*0.2, y=900 + i*20)
            e.config(text='%s' % final_list[j][i])
            cols_list.append(e)
        rows_list.append(cols)

    # this section of code is for tool-tipping the scheduled procedures
    # if date correspond to date of scheduled surgery, make tool-tip appear
    for i in range(1, len(final_list[1])):
        heading_text = heading.cget('text')
        heading_parts = heading_text.split(' , ')
        heading_date = str(datetime.strptime(heading_parts[1], '%Y-%m-%d').date())
        list_date = str(datetime.strptime(final_list[1][i], '%d/%m/%Y').date())
        if list_date == heading_date:
            starting_time = final_list[3][i].split(' - ')[0]
            start_hour, start_min = starting_time.split(':')
            starting_slot = int((int(start_hour)+int(start_min)/60)/0.25 - 32)
            slot_numbers = int(float(final_list[4][i].split(' ')[0])/0.25)
            ot_index = ot_list.index(final_list[2][i])
            for j in range(slot_numbers):
                text = final_list[0][0] + ' ' + final_list[0][i] + '\n' + \
                       final_list[1][0] + ' ' + final_list[1][i] + '\n' + \
                       final_list[2][0] + ' ' + final_list[2][i] + '\n' + \
                       final_list[3][0] + ' ' + final_list[3][i]
                tip = Balloon(canvas_frame)
                tip.bind_widget(blk[starting_slot+j][ot_index], balloonmsg=text)

    def FrameWidth(event):
        canvas_width = event.width
        canvas.itemconfig(canvas_window, width=canvas_width)

    def OnFrameConfigure(event):
        canvas.configure(scrollregion=canvas.bbox("all"))

    canvas_frame.bind("<Configure>", OnFrameConfigure)
    canvas.bind('<Configure>', FrameWidth)

    root.mainloop()


# current_schedules = read_csv()
# show_timetable(current_schedules)


import os
import csv
from tkinter import *
from tkinter import filedialog
from datetime import *
from view_timetable_old import show_timetable


def read_csv():

    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    # this will obtain file path of the selected csv file
    file_path = filedialog.askopenfilename(title='Please select the schedule .csv file')
    file_path = os.path.normpath(file_path)

    current_schedules = {}  # this will be the final output consisting of all the required information
    current_dates = []  # this will store the unique dates in the csv i.e 17/11/2020, 18/11/2020
    temp_dict = {}  # this temp dictionary will store the dictionaries, day1={}, day2={} ...
    temp_days = []  # this temp list will number of days from now for each unique date, 3, 4 ...

    # this is to find out the number of unique days in the csv, and create a tuple for each unique day
    # with open (r'C:\Users\User\Desktop\mockData.csv', newline='') as csv_file:
    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv:  # get unique dates in the csv file
            items = row[1].split(' ')
            if items[0] not in current_dates:
                current_dates.append(items[0])

    # for each day in current_dates, create a dictionary i.e day4 = {}
    for i in current_dates:
        selected_date = datetime.strptime(i, '%d/%m/%Y').date()
        days_from_now = (selected_date - date.today()).days
        temp_days.append(days_from_now)  # the days from today for each day_dictionary is stored in temp_days
        temp_dict['day%s' % days_from_now] = {}  # the day_dictionaries are stored in temp_dict

    # create a key for each operating theatre, in each day_dictionary
    for i in range(len(current_dates)):
        (temp_dict['day' + str(temp_days[i])])['L1'] = []
        (temp_dict['day' + str(temp_days[i])])['L2'] = []
        (temp_dict['day' + str(temp_days[i])])['L3'] = []
        (temp_dict['day' + str(temp_days[i])])['L4'] = []
        (temp_dict['day' + str(temp_days[i])])['L5'] = []
        (temp_dict['day' + str(temp_days[i])])['L6'] = []
        (temp_dict['day' + str(temp_days[i])])['L7'] = []
        (temp_dict['day' + str(temp_days[i])])['L8'] = []
        (temp_dict['day' + str(temp_days[i])])['M1'] = []
        (temp_dict['day' + str(temp_days[i])])['M2'] = []
        (temp_dict['day' + str(temp_days[i])])['M3'] = []
        (temp_dict['day' + str(temp_days[i])])['M4'] = []
        (temp_dict['day' + str(temp_days[i])])['M5'] = []
        (temp_dict['day' + str(temp_days[i])])['OT 24'] = []
        (temp_dict['day' + str(temp_days[i])])['OT 25'] = []
        (temp_dict['day' + str(temp_days[i])])['OT 22'] = []
        (temp_dict['day' + str(temp_days[i])])['R1'] = []
        (temp_dict['day' + str(temp_days[i])])['R4'] = []
        (temp_dict['day' + str(temp_days[i])])['R5'] = []
        (temp_dict['day' + str(temp_days[i])])['R6'] = []
        (temp_dict['day' + str(temp_days[i])])['R7'] = []
        (temp_dict['day' + str(temp_days[i])])['R8'] = []
        (temp_dict['day' + str(temp_days[i])])['MRI'] = []

    # read each line in csv and append details to the respective 'OT' dictionaries
    with open (file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        # skip header
        next(mock_csv, None)
        for row in mock_csv:
            start_date = row[1].split(' ')
            end_date = row[2].split(' ')
            start_time = start_date[1]
            end_time = end_date[1]
            # codes to convert time into appropriate format 08:15 = 8.25, 10:45 = 10.75
            start_time_new = datetime.strptime(start_time, '%H:%M').time()
            end_time_new = datetime.strptime(end_time, '%H:%M').time()
            start_time_deci = (start_time_new.hour+start_time_new.minute/60.0)
            end_time_deci = (end_time_new.hour + end_time_new.minute / 60.0)
            duration = end_time_deci - start_time_deci  # calculate duration of surgery procedure
            date_index = current_dates.index(start_date[0])
            # print(temp_dict['day'+str(temp_days[date_index])]) # this will call the respective day_dictionary
            # this is to append list into the OT dictionaries
            (temp_dict['day' + str(temp_days[date_index])])[row[5]].append([start_time_deci, end_time_deci, duration, row[4], False, False])

    d1 = '2020-11-23'
    d1 = datetime.strptime(d1, '%Y-%m-%d').date()
    d2 = '2020-11-24'
    d2 = datetime.strptime(d2, '%Y-%m-%d').date()
    d3 = '2020-11-25'
    d3 = datetime.strptime(d3, '%Y-%m-%d').date()
    d4 = '2020-11-26'
    d4 = datetime.strptime(d4, '%Y-%m-%d').date()
    d5 = '2020-11-27'
    d5 = datetime.strptime(d5, '%Y-%m-%d').date()

    week_dict = {0:1, 7:2, 14:3, 21:4, 28:5}

    for i in temp_days:  # this will load all the necessary days into the current_schedule dictionary
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Monday':
            temp_date = (((date.today() + timedelta(days=i))-d1).days)%35
            current_schedules[i] = (temp_dict['day' + str(i)], (date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Tuesday':
            temp_date = (((date.today() + timedelta(days=i))-d2).days)%35
            current_schedules[i] = (temp_dict['day' + str(i)], (date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Wednesday':
            temp_date = (((date.today() + timedelta(days=i))-d3).days)%35
            current_schedules[i] = (temp_dict['day' + str(i)], (date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Thursday':
            temp_date = (((date.today() + timedelta(days=i))-d4).days)%35
            current_schedules[i] = (temp_dict['day' + str(i)], (date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Friday':
            temp_date = (((date.today() + timedelta(days=i))-d5).days)%35
            current_schedules[i] = (temp_dict['day' + str(i)], (date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))

    ot_list = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'M1', 'M2', 'M3', 'M4', 'M5', 'OT 24',
               'OT 25', 'OT 22', 'R1', 'R4', 'R5', 'R6', 'R7', 'R8', 'MRI']

    # access all OT for each day
    for i in current_schedules:
        for j in ot_list:
            if len(current_schedules[i][0][j]) == 0:  # check if each OT_dictionary is empty
                current_schedules[i][0][j].append([8.0, 19.0, 11.0, 'empty', False, False])
            if len(current_schedules[i][0][j]) != 0 and current_schedules[i][0][j][0][
                0] != 8.0:  # add 'empty' from 8am to start of first procedure
                current_schedules[i][0][j].insert(0, [8.0, current_schedules[i][0][j][0][0], current_schedules[i][0][j][0][0] - 8.0, 'empty', False, False])
            if len(current_schedules[i][0][j]) != 0 and current_schedules[i][0][j][-1][
                1] != 19.0:  # add 'empty' from end of last procedure to 7pm
                current_schedules[i][0][j].append(
                    [current_schedules[i][0][j][-1][1], 19.0, 19.0 - current_schedules[i][0][j][-1][1], 'empty', False, False])

    # add 'empty' for empty timeslots in between each procedure allocated
    for i in current_schedules:
        for j in ot_list:
            if current_schedules[i][0][j] != 1:
                for k in range(len(current_schedules[i][0][j])):
                    # this will check if the end time of the preceding procedure = start time of next procedure
                    # this will also make sure that it is not the last item in the list
                    if current_schedules[i][0][j][k] != current_schedules[i][0][j][-1] and current_schedules[i][0][j][k][1] != current_schedules[i][0][j][k + 1][0]:
                        current_schedules[i][0][j].insert(k + 1, [current_schedules[i][0][j][k][1], current_schedules[i][0][j][k + 1][0],
                                                           current_schedules[i][0][j][k + 1][0] - current_schedules[i][0][j][k][1],
                                                           'empty', False, False])

    return current_schedules


output = read_csv()
show_timetable(output)


import numpy as np
import random
import Generate_initial_solution as gs
import copy
import pareto as pt
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit


def pareto_ranking(scores1, solns, n):
	scores = copy.deepcopy(scores1)
	pareto_rankings = {}
	pareto_sorted_scores = []
	pareto_sorted_soln = []
	rank = 0
	n_pop = 0
	while len(scores) != 0:
		pareto_efficient_pts = pt.eps_sort(scores)
		l = len(pareto_efficient_pts)
		n_pop += l
		if n_pop >= n:
			overshot = n_pop-n
			pareto_efficient_pts = pareto_efficient_pts[0:(l-overshot)]
			pareto_rankings[rank] = pareto_efficient_pts
			pareto_sorted_scores = pareto_sorted_scores + pareto_efficient_pts
			scores = [x for x in scores if x not in pareto_efficient_pts]
			rank += 1
			for i in pareto_efficient_pts:
				loc = np.where((scores1 == np.array(i)).all(axis=1))[0][0]
				pareto_sorted_soln.append(solns[loc])
			return pareto_rankings, pareto_sorted_scores, pareto_sorted_soln
		pareto_rankings[rank] = pareto_efficient_pts
		pareto_sorted_scores = pareto_sorted_scores + pareto_efficient_pts
		scores = [x for x in scores if x not in pareto_efficient_pts]
		rank += 1
		for i in pareto_efficient_pts:
			loc = np.where((scores1 == np.array(i)).all(axis=1))[0][0]
			pareto_sorted_soln.append(solns[loc])
	return pareto_rankings, pareto_sorted_scores, pareto_sorted_soln


def pareto_score_soln_sore(scores1, soln):
	scores = copy.deepcopy(scores1)
	pareto_sorted_scores = []
	pareto_sorted_soln = []


def select_two_parent(ranks, list_scores, list_soln):
	# roulette wheel to select parent
	dict_len = len(ranks)
	frac = np.flip(np.arange(start=1, stop=dict_len+1))
	space = np.sum(frac)
	frac = frac/space
	cum_frac = np.cumsum(frac)
	r1 = random.uniform(0, 1)
	r2 = random.uniform(0, 1)
	rank_chosen1 = np.argwhere(cum_frac>r1)[0][0]
	rank_chosen2 = np.argwhere(cum_frac>r2)[0][0]
	p1_score = random.choice(ranks[rank_chosen1])
	p2_score = random.choice(ranks[rank_chosen2])
	p1_score_loc = np.where((list_scores==np.array(p1_score)).all(axis=1))[0][0]
	p2_score_loc = np.where((list_scores==np.array(p2_score)).all(axis=1))[0][0]
	return list_soln[p1_score_loc], list_soln[p2_score_loc]


def GA_crossover(p1, p2, sch_list, empty_schedule):
	child = copy.deepcopy(empty_schedule)
	clashes = []
	for to_sch in sch_list:
		r = random.randint(1, 2)
		surg = to_sch[0]
		if r == 1:
			p1_surg_loc = np.argwhere(p1==surg)
			if check_if_zero(child,p1_surg_loc):
				#put it in
				day = p1_surg_loc[0][0]
				OT = p1_surg_loc[0][1]
				start_ts = p1_surg_loc[0][2]
				end_ts = p1_surg_loc[-1][2] + 1
				np.put(child[day][OT], np.arange(start=start_ts, stop=end_ts), [surg])
			else:
				clashes.append(to_sch)
		elif r == 2:
			p2_surg_loc = np.argwhere(p2==surg)
			if check_if_zero(child,p2_surg_loc):
				#put it in
				day = p2_surg_loc[0][0]
				OT = p2_surg_loc[0][1]
				start_ts = p2_surg_loc[0][2]
				end_ts = p2_surg_loc[-1][2] + 1
				np.put(child[day][OT], np.arange(start=start_ts, stop=end_ts), [surg])
			else:
				clashes.append(to_sch)
	#print(clashes)
	return child, clashes


def GA_mutate(child, clashes, MAS_allowed, OT_indexes, day_of_week_indexes):
	if len(clashes)==0:
		return child
	else:
		child = initial_solution_random_fit(child, clashes, MAS_allowed, day_of_week_indexes, OT_indexes)
		return child


def check_if_zero(empty_schedule, locs):
	len_zero = len(locs)
	day = locs[0][0]
	OT = locs[0][1]
	start_ts = locs[0][2]
	end_ts = locs[-1][2] + 1
	partition = empty_schedule[day][OT][start_ts:end_ts]
	if np.all(partition == np.zeros(len_zero)):
		return True
	else:
		return False


from tkinter import *
from PIL import ImageTk,Image
from tkinter import filedialog
import os
from test_hco import run_hco
from test_ts import run_ts
from test_sa import run_sa
from read_MAS import read_mas1
from read_csv import read_csv1
from read_toschedule import read_toschedule1
from view_timetable import show_timetable
from threading import Thread

def register_user():
    username_info = username.get()
    password_info = password.get()

    file=open(username_info, 'w')
    file.write(username_info+'\n')
    file.write(password_info)
    file.close()

    username_entry.delete(0, END)
    password_entry.delete(0, END)

    Label(screen1, text='Registration success', fg='green').pack()


def register():
    global screen1
    screen1 = Toplevel(screen)
    screen1.title('Register')
    screen1.geometry("400x250+750+350")

    global username
    global password
    global username_entry
    global password_entry
    username = StringVar()
    password = StringVar()

    Label(screen1, text='Please enter details below').pack()
    Label(screen1, text='').pack()
    Label(screen1, text='Username *').pack()
    username_entry = Entry(screen1, textvariable=username)
    username_entry.pack()
    Label(screen1, text='Password *').pack()
    password_entry = Entry(screen1, textvariable=password)
    password_entry.pack()
    Label(screen1, text='').pack()
    Button(screen1, text='Register', width=10, height=2, command=register_user).pack()


def login_button():
    global screen2
    screen2 = Tk()
    screen2.title('Please select optimizer and corresponding csv files')
    screen2.geometry("1200x700+350+150")

    # frame 2 and the widgets
    frame2 = Frame(screen2)
    frame2.place(relx=0, rely=0, relwidth=0.495, relheight=1)

    label4 = Label(frame2, text='Step 1 :')
    label4.place(relx=0, rely=0)
    label4.config(font=('', 20))

    label5 = Label(frame2, text='Select the Current Schedule .csv file :')
    label5.place(relx=0.3, rely=0.32)
    label5.config(font=('', 10))

    entry1 = Entry(frame2, width=30)
    entry1.place(relx=0.3, rely=0.35)

    def browsefunc1():
        global file_path_1

        file_name = filedialog.askopenfilename(title='Please select the schedule .csv file')
        file_path_1 = os.path.normpath(file_name)

        head, tail = os.path.split(file_name)
        # file_path = os.path.normpath(file_name)
        entry1.delete(0, 'end')
        entry1.insert(0, tail)  # add this

    b1 = Button(frame2, text="Browse :", command=browsefunc1)
    b1.place(relx=0.65, rely=0.35)

    label6 = Label(frame2, text='Select the to be Scheduled .csv file :')
    label6.place(relx=0.3, rely=0.42)
    label6.config(font=('', 10))

    entry2 = Entry(frame2, width=30)
    entry2.place(relx=0.3, rely=0.45)

    def browsefunc2():
        global file_path_2
        file_name = filedialog.askopenfilename(title='Please select the to be scheduled .csv file')
        file_path_2 = os.path.normpath(file_name)

        head, tail = os.path.split(file_name)
        entry2.delete(0, 'end')
        entry2.insert(0, tail)  # add this

    b2 = Button(frame2, text="Browse :", command=browsefunc2)
    b2.place(relx=0.65, rely=0.45)

    label7 = Label(frame2, text='Select the MAS .csv file :')
    label7.place(relx=0.3, rely=0.52)
    label7.config(font=('', 10))

    entry3 = Entry(frame2 , width=30)
    entry3.place(relx=0.3, rely=0.55)

    def browsefunc3():
        global file_path_3
        file_name = filedialog.askopenfilename(title='Please select the MAS .csv file')
        file_path_3 = os.path.normpath(file_name)

        head, tail = os.path.split(file_name)
        entry3.delete(0, 'end')
        entry3.insert(0, tail)  # add this

    b3 = Button(frame2, text="Browse :", command=browsefunc3)
    b3.place(relx=0.65, rely=0.55)

    divider_frame2 = Frame(screen2, bg='grey')
    divider_frame2.place(relx=0.495, rely=0, relwidth=0.005, relheight=1)

    # frame 3 and its widgets
    frame3 = Frame(screen2)
    frame3.place(relx=0.5, rely=0, relwidth=0.5, relheight=1)

    label7 = Label(frame3, text='Step 2 :')
    label7.place(relx=0, rely=0)
    label7.config(font=('', 20))

    label8 = Label(frame3, text='Select Optimizer :')
    label8.place(relx=0.35, rely=0.3)
    label8.config(font=('', 10))

    elements = ['Hill Climbing', 'Tabu Search', 'Simulated Annealing', 'Genetic Algorithm']

    list_box = Listbox(frame3, selectmode=SINGLE)
    list_box.insert(END, *elements)
    list_box.place(relx=0.35, rely=0.33)

    def generate_schedule():

        current_schedule = read_csv1(file_path_1)
        to_schedule = read_toschedule1(file_path_2)
        mas_full = read_mas1(file_path_3)

        optimizer_selected = list_box.get(list_box.curselection())

        optimizer_dict = {'Hill Climbing': 1, 'Tabu Search': 2, 'Simulated Annealing': 3,
                          'Genetic Algorithm': 4}

        if optimizer_dict[optimizer_selected] == 1:
            run_hco(current_schedule, to_schedule, mas_full)
        if optimizer_dict[optimizer_selected] == 2:
            run_ts(current_schedule, to_schedule, mas_full)
        if optimizer_dict[optimizer_selected] == 3:
            run_sa(current_schedule, to_schedule, mas_full)
        if optimizer_dict[optimizer_selected] == 4:
            run_ga(current_schedule, to_schedule, mas_full)

    b4 = Button(frame3, text='Generate Schedule', command=generate_schedule)
    b4.place(relx=0.37, rely=0.65)


def main_screen():
    global screen
    screen = Tk()
    screen.geometry("1200x700+350+150")
    screen.title("Scheduling Application")
    Label(text='Singhealth Meta-heuristics Operating Theatre Scheduling Optimization', bg='#D35400', width='1200', height='2').pack()
    load = Image.open('C:/Users/User/Desktop/singhealth_background.png')
    render = ImageTk.PhotoImage(load)
    img = Label(screen, image= render)
    img.place(relx=0.99, rely=0.98, anchor=SE)

    username_verify = StringVar()
    password_verify = StringVar()

    # username heading and entry
    u_label = Label(screen, text='Username :')
    u_label.place(relx=.5, rely=.32, anchor="center")
    username_entry_main = Entry(screen, textvariable=username_verify, width=30)
    username_entry_main.place(relx=.5, rely=.35, anchor="center")

    # password heading and entry
    p_label = Label(screen, text='Password :')
    p_label.place(relx=.5, rely=.42, anchor="center")
    password_entry_main = Entry(screen, textvariable=password_verify, width=30)
    password_entry_main.place(relx=.5, rely=.45, anchor="center")

    def clear_frame():
        screen.destroy()
        login_button()

    # login and register buttons
    Button(text='Login', height='1', width='10', command=clear_frame).place(relx=.5, rely=.5, anchor="center")
    Button(text='Register', height='1' ,width='10', command=register).place(relx=.5, rely=.55, anchor="center")


    screen.mainloop()


main_screen()


import random
import numpy as np
import copy
import swapper as sp


def zero_runs(a):
	# Create an array that is 1 where a is 0, and pad each end with an extra 0.
	iszero = np.concatenate(([0], np.equal(a, 0).view(np.int8), [0]))
	absdiff = np.abs(np.diff(iszero))
	# Runs start and end where absdiff is 1.
	r = np.where(absdiff==1)[0]
	ranges = np.where(absdiff == 1)[0].reshape(-1, 2)
	return ranges


def zero_runs_3d(a):
	iszero = np.equal(a, 0)
	npad = ((0, 0), (0, 0), (1, 1))
	iszero = np.pad(iszero, pad_width=npad, mode='constant', constant_values=0).view(np.int8)
	absdiff = np.abs(np.diff(iszero, axis=2))
	ranges = np.argwhere(absdiff == 1)
	even = ranges[::2]
	odd_c = ranges[1::2][:, 2:]
	return np.hstack((even, odd_c))


def initial_solution_first_fit(current_schedule, to_schedule, MAS_allowed, day_of_week_indexes, OT_indexes):
	for slot in to_schedule:
		flag = False
		for index_days, days in enumerate(current_schedule):
			current_day = day_of_week_indexes[index_days]
			for index_OT, OT in enumerate(days):
				if not OT_indexes[index_OT] in MAS_allowed[current_day]:
					continue
				zero_index = zero_runs(OT)
				zero_index_diff = np.diff(zero_index, axis=1) # length of 0 slots
				bool_vec = len(slot) <= zero_index_diff
				if np.any(bool_vec):
					legal_slot_index = np.argmax(bool_vec, axis=0)
					l = zero_index[legal_slot_index][0, 0]
					r = l + len(slot)
					OT[l:r] = slot
					flag = True
					break
			if flag:
				break
		if not flag:
			slot_dur = len(slot) * 15
			print(str(slot[0]) + " of " + str(
				slot_dur) + " mins has no possible arrangements, extend horizon or reduce number of cases to schedule")
	return current_schedule


def initial_solution_random_fit(current_schedule, ts, MAS_allowed, day_of_week_indexes, OT_indexes):
	to_schedule = copy.deepcopy(ts)
	curr_sche = copy.deepcopy(current_schedule)
	while len(to_schedule) != 0:
		zero_index = zero_runs_3d(curr_sche)
		z = np.hstack((zero_index[:, :2], np.diff(zero_index[:, 2:], axis=1)))
		slot_index = random.choice(range(len(to_schedule)))
		slot = to_schedule[slot_index]
		slot_len = len(slot)
		r = [1 if (OT_indexes[i[1]] in MAS_allowed[day_of_week_indexes[i[0]]] and i[2] > slot_len) else 0 for i in z]
		idx_nonzero = np.nonzero(r)
		#print(len(idx_nonzero[0]))
		if len(idx_nonzero[0]) == 0:
			print("no slot for surgery " + str(slot[0]) + ", try extending horizon or changing to use first fit")
			to_schedule.pop(slot_index)
			continue
		random_pick_legal_slot = np.random.choice(idx_nonzero[0])  # random choice of a slot that is legal
		day_chosen = zero_index[random_pick_legal_slot][0]
		OT_chosen = zero_index[random_pick_legal_slot][1]
		chosen_start = zero_index[random_pick_legal_slot][2]
		chosen_end = zero_index[random_pick_legal_slot][3]
		np.put(curr_sche[day_chosen][OT_chosen], np.arange(start=chosen_start, stop=chosen_start + slot_len),
		       slot[0])
		to_schedule.pop(slot_index)
	return curr_sche


from setuptools import setup, find_packages

classifiers = [
	'Development Status :: 5 - Production/Stable',
	'Intended Audience :: Education',
	'Operating System :: Microsoft :: Windows :: Windows 10',
	'License :: OSI Approved :: MIT License',
	'Programming Language :: Python :: 3'
]

setup(
	name='SDP18',
	version='0.1.0',
	description='This package schedules surgeries using metaheurisitcs. It is a project done for in NUS for Systems Design Project (SDP):Metaheuristic Surgery Scheduling for Operating Theatre Scheduling',
	long_description=open('README.md').read(),
	url='https://github.com/lwq96/NewProject',
	author='SDP18',
	author_email='e0176071@u.nus.edu',
	license='MIT',
	classifiers=classifiers,
	keywords='metaheuristic, surgery, scheduling',
	packages=find_packages(),
	install_requires=['']
)


import numpy as np
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit
from MO_fitting_func_calc import MO_calculate
from MO_fitting_func_calc import calculate_delta_S
from MO_fitting_func_calc import calc_crowding_distance
import find_legal_neighbors as fd
import swapper as sp
import pareto as pt
from read_MAS import read_mas
from read_csv import read_csv
from read_toschedule import read_toschedule
from plot_pareto_front import plot_front
import time
import random
from view_timetable1 import show_timetable
from threading import Thread
from output_table import create_table


def run_sa(current_schedule_list, to_schedule_read, mas_read):

    random.seed(10)

    OT_indexes = {0: 'L1',
                  1: 'L2',
                  2: 'L3',
                  3: 'L4',
                  4: 'L5',
                  5: 'L6',
                  6: 'L7',
                  7: 'L8',
                  8: 'M1',
                  9: 'M2',
                  10: 'M3',
                  11: 'M4',
                  12: 'M5',
                  13: 'OT 24',
                  14: 'OT 25',
                  15: 'OT 22',
                  16: 'R1',
                  17: 'R4',
                  18: 'R5',
                  19: 'R6',
                  20: 'R7',
                  21: 'R8',
                  22: 'MRI',
                  }

    current_schedule = current_schedule_list
    # add one more schedule_date
    schedule_date = current_schedule[2]
    # code on top ^
    schedule_days = current_schedule[1]
    day_index = current_schedule[3]
    current_schedule = current_schedule[0]
    to_schedule = to_schedule_read
    MAS_full = mas_read
    MAS_allowed = MAS_full['OTO']

    t0 = time.time()

    # first fit or random fit
    soln = initial_solution_random_fit(current_schedule, to_schedule, MAS_allowed, day_index, OT_indexes)
    # SIMULATED ANNEALING ALGORITHM
    print("SIMULATED ANNEALING Algorithm")
    T = 1000  # higher temperature, higher chance of accepting bad solution (more diversed solution)
    alpha = 0.99  # cooling rate
    termination_number = 30  # how many times of reaching the same fitting value to stop the run?
    termination_count = 0

    while termination_count < termination_number:
        sp.pushback2(soln)
        curr_score = MO_calculate(soln)
        legal_swaps = fd.all_legal_swaps(soln, to_schedule, MAS_allowed, day_index, OT_indexes)
        neighbour = random.choice(legal_swaps)
        perturb_soln = sp.swapping(soln, neighbour)
        perturb_score = MO_calculate(perturb_soln)
        before = curr_score
        if np.all(perturb_score <= curr_score):
            soln = perturb_soln
            curr_score = perturb_score
        else:  # if does not dominate, select based on some probability, UMOSA
            p = calculate_delta_S(soln, perturb_soln, T)
            if p > random.uniform(0, 1):
                soln = perturb_soln
                curr_score = perturb_score

        # check improvement for termination
        improvement = before - curr_score
        if np.all(improvement==0):
            termination_count += 1
        else:
            termination_count = 0

        print("The current score is:" + str(curr_score))
        T = T * alpha

    t1 = time.time()
    print("Time Taken for SA: " + str(t1 - t0) + ' seconds')
    # sol_list = [soln, schedule_days]
    # thread1 = Thread(target=show_timetable, args=[sol_list])
    # thread1.start()
    # thread2 = Thread(target=create_table, args=(soln, schedule_date))
    # thread2.start()
    sol_list = [soln, schedule_days, schedule_date]
    thread1 = Thread(target=show_timetable, args=[sol_list])
    thread1.start()
    print(curr_score)


import numpy as np
import copy
import time

def swapping(soln, swap):

	temp_soln = copy.deepcopy(soln)
	swap1name = swap[0][0]
	swap1day = swap[0][1]
	swap1OT = swap[0][2]
	swap1start = swap[0][3]
	swap1end = swap[0][4]
	swap2name = swap[1][0]
	swap2day = swap[1][1]
	swap2OT = swap[1][2]
	swap2start = swap[1][3]
	swap2end = swap[1][4]
	if swap1name == 0 or swap2name == 0:

		if swap1name == 0:
			np.put(temp_soln[swap1day][swap1OT], np.arange(start=swap1start, stop=(swap1start+(swap2end-swap2start))), swap2name)
			np.put(temp_soln[swap2day][swap2OT], np.arange(start=swap2start, stop=swap2end), 0)
		elif swap2name == 0:
			np.put(temp_soln[swap2day][swap2OT], np.arange(start=swap2start, stop=(swap2start+(swap1end-swap1start))), swap1name)
			np.put(temp_soln[swap1day][swap1OT], np.arange(start=swap1start, stop=swap1end), 0)
	elif swap1name != 0 and swap2name != 0:

		block1 = copy.deepcopy(temp_soln[swap1day][swap1OT][swap1start:swap1end])
		block2 = copy.deepcopy(temp_soln[swap2day][swap2OT][swap2start:swap2end])
		len1 = len(block1)
		len2 = len(block2)
		len_diff = abs(len1-len2)
		if len1 > len2:
			block2 = np.hstack(block2, [0] * len_diff)
			temp_soln[swap1day][swap1OT][swap1start:swap1end] = block2
			temp_soln[swap2day][swap2OT][swap2start:(swap2end+len_diff)] = block1
		elif len1 < len2:
			block1 = np.hstack(block1, [0] * (len1-len2))
			temp_soln[swap2day][swap2OT][swap2start:swap2end] = block1
			temp_soln[swap1day][swap1OT][swap1start:(swap1end+len_diff)] = block2
		else: #equal length
			temp_soln[swap1day][swap1OT][swap1start:swap1end] = block2
			temp_soln[swap2day][swap2OT][swap2start:swap2end] = block1
	return temp_soln



def pushback2(soln):
	temp_ = copy.deepcopy(soln)
	temp_[temp_ > 2000000] = 9999999
	temp_diff = np.diff(temp_, axis=2)
	indices = (temp_diff > 1100000) & (temp_diff < 1300000)  # find indexes where its empty before a swappable surgery
	match = np.argwhere(indices==True)
	for index_match in match:
		day = index_match[0]
		OT = index_match[1]
		index_of_first_0_end = index_match[2] + 1
		index_of_first_0_start = trailzero(soln[day][OT], index_of_first_0_end - 1)
		index_of_surg_end = trialsurg(soln[day][OT], index_of_first_0_end + 1)
		surg_code = soln[day][OT][index_of_first_0_end + 1]
		trailing0_len = index_of_first_0_end - index_of_first_0_start
		index_of_surg_replace_start = index_of_surg_end - trailing0_len
		np.put(soln[day][OT], np.arange(start=index_of_first_0_start, stop=index_of_first_0_end), [surg_code])
		np.put(soln[day][OT], np.arange(start=index_of_surg_replace_start, stop=index_of_surg_end), [0])
	# check if its done else re do
	temp_ = copy.deepcopy(soln)
	temp_[temp_ > 2000000] = 9999999
	temp_diff = np.diff(temp_, axis=2)
	indices = (temp_diff > 1100000) & (temp_diff < 1300000)  # find indexes where its empty before a swappable surgery
	match = np.argwhere(indices == True)
	if len(match) != 0:
		print("re-pushback")
		pushback2(soln)
	return


def trialsurg(a, b):
	surg_code = a[b]
	while a[b] == surg_code:
		b = b + 1
		if b == 44:
			return b

	return b


def trailzero(a, b):

	while a[b] == 0:
		b = b-1
		if b == 0:
			return b
	return b + 1


import numpy as np
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit
from MO_fitting_func_calc import MO_calculate
from MO_fitting_func_calc import calculate_delta_S
from MO_fitting_func_calc import calc_crowding_distance
import find_legal_neighbors as fd
import swapper as sp
import pareto as pt
from read_MAS import read_mas
from read_csv import read_csv
from read_toschedule import read_toschedule
import plot_pareto_front as pf
import time
import random
from view_timetable1 import show_timetable
from threading import Thread
from output_table import create_table

def run_ts(current_schedule_list, to_schedule_read, mas_read):

    OT_indexes = {0: 'L1',
                  1: 'L2',
                  2: 'L3',
                  3: 'L4',
                  4: 'L5',
                  5: 'L6',
                  6: 'L7',
                  7: 'L8',
                  8: 'M1',
                  9: 'M2',
                  10: 'M3',
                  11: 'M4',
                  12: 'M5',
                  13: 'OT 24',
                  14: 'OT 25',
                  15: 'OT 22',
                  16: 'R1',
                  17: 'R4',
                  18: 'R5',
                  19: 'R6',
                  20: 'R7',
                  21: 'R8',
                  22: 'MRI',
                  }
    current_schedule = current_schedule_list
    # add one more schedule_date
    schedule_date = current_schedule[2]
    # code on top ^
    schedule_days = current_schedule[1]
    day_index = current_schedule[3]
    current_schedule = current_schedule[0]
    to_schedule = to_schedule_read
    MAS_full = mas_read
    MAS_allowed = MAS_full['OTO']

    t0 = time.time()
    # first fit or random fit
    k = 5  # number of parallel searches
    tabu_tenure = 5
    soln_list = []
    for _ in range(k):
        soln_list.append(
            initial_solution_random_fit(current_schedule, to_schedule, MAS_allowed, day_index, OT_indexes))

    pareto_front = np.array([[9999, 9999, 9999]])
    max_iter = 50
    tabu_list = [[None] for i in range(k)]

    # PARALLEL TABU SEARCH ALGORITHM
    print("TABU SEARCH Algorithm")
    # Trying to esimate pareto front from multiple initial solutions

    for iter in range(max_iter):
        new_search_front_scores = []
        new_search_front = []
        scores_t = np.array([[9999, 9999, 9999]])
        for index, soln in enumerate(soln_list):
            sp.pushback2(soln)
            curr_score = MO_calculate(soln)
            # generate all legal swaps
            legal_swaps = fd.all_legal_swaps_subset(soln, to_schedule, MAS_allowed, day_index,
                                                    OT_indexes)  # use a random subsetted neighbour to make program run faster
            scores = np.array([[9999, 9999, 9999]])
            for swaps in legal_swaps:  # bottleneck
                sc = MO_calculate(sp.swapping(soln, swaps))
                scores = np.vstack((scores, sc))
            scores = np.delete(scores, 0, axis=0)
            scores_t = np.concatenate((scores_t, scores))
            p_front_scores_x = pf.pareto_ranking(
                scores)  # not really pareto front, with some that are not on pareto front, but also quite good
            pareto_front = np.concatenate((pareto_front, p_front_scores_x), axis=0)  # will have to squeeze later

            c = random.choice(p_front_scores_x)
            c_index = np.where((scores == c).all(axis=1))[0][0]
            c_soln = sp.swapping(soln, legal_swaps[c_index])
            while c in tabu_list[index]:
                print('tabu!')
                c = random.choice(p_front_scores_x)
                c_index = np.where((scores == c).all(axis=1))[0][0]
                c_soln = sp.swapping(soln, legal_swaps[c_index])
            new_search_front_scores.append(c)  # the next iteration's soln list
            new_search_front.append(c_soln)
            # update tabu list
            tabu_list[index].pop(0)
            tabu_list[index].extend(c * tabu_tenure)
        scores_t = np.delete(scores_t, 0, axis=0)
        print(new_search_front_scores)
        soln_list = new_search_front

    pareto_front = np.delete(pareto_front, 0, axis=0)
    pareto_front_final = np.array(pt.eps_sort(pareto_front))
    print("Pareto Front Scores:")
    print(pareto_front_final)

    # find minimum overtime score
    min_over_timescore = pareto_front_final[np.argmin(pareto_front_final[:, 0]), :]
    print("Minimum Overtime Score")
    print(min_over_timescore)
    t1 = time.time()
    print("Time Taken for TS: " + str(t1 - t0) + ' seconds')

    # sol_list = [soln, schedule_days]
    # thread1 = Thread(target=show_timetable, args=[sol_list])
    # thread1.start()
    # thread2 = Thread(target=create_table, args=(soln, schedule_date))
    # thread2.start()
    sol_list = [soln, schedule_days, schedule_date]
    thread1 = Thread(target=show_timetable, args=[sol_list])
    thread1.start()

    pf.plot_front(scores, pareto_front_final)